@@ -1,6 +1,6 @@
 /* unit.c API unit tests driver
  *
- * Copyright (C) 2006-2020 wolfSSL Inc.
+ * Copyright (C) 2006-2021 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -30,6 +30,7 @@
 
 #include <stdio.h>
 #include <tests/unit.h>
+#include <wolfssl/wolfcrypt/fips_test.h>
 
 
 int myoptind = 0;
@@ -52,7 +53,7 @@ int unit_test(int argc, char** argv)
 
 #ifdef WOLFSSL_FORCE_MALLOC_FAIL_TEST
     if (argc > 1) {
-        word32 memFailCount = atoi(argv[1]);
+        int memFailCount = atoi(argv[1]);
         printf("\n--- SET RNG MALLOC FAIL AT %d---\n", memFailCount);
         wolfSSL_SetMemFailCount(memFailCount);
     }
@@ -64,6 +65,9 @@ int unit_test(int argc, char** argv)
     wolfSSL_Debugging_ON();
 #endif
 
+#ifdef WC_RNG_SEED_CB
+    wc_SetSeed_Cb(wc_GenerateSeed);
+#endif
 #ifdef HAVE_WNR
     if (wc_InitNetRandom(wnrConfig, NULL, 5000) != 0)
         err_sys("Whitewood netRandom global config failed");
@@ -73,13 +77,95 @@ int unit_test(int argc, char** argv)
     ChangeToWolfRoot();
 #endif
 
-    ApiTest();
+#if defined(HAVE_FIPS) && defined(HAVE_FIPS_VERSION) && (HAVE_FIPS_VERSION == 5)
+#if !defined(NO_AES) && !defined(NO_AES_CBC)
+    if (wc_RunCast_fips(FIPS_CAST_AES_CBC) != 0) {
+        err_sys("AES-CBC CAST failed");
+    }
+#endif
+#ifdef HAVE_AESGCM
+    if (wc_RunCast_fips(FIPS_CAST_AES_GCM) != 0) {
+        err_sys("AES-GCM CAST failed");
+    }
+#endif
+#ifndef NO_SHA
+    if (wc_RunCast_fips(FIPS_CAST_HMAC_SHA1) != 0) {
+        err_sys("HMAC-SHA1 CAST failed");
+    }
+#endif
+    /* the only non-optional CAST */
+    if (wc_RunCast_fips(FIPS_CAST_HMAC_SHA2_256) != 0) {
+        err_sys("HMAC-SHA2-256 CAST failed");
+    }
+#ifdef WOLFSSL_SHA512
+    if (wc_RunCast_fips(FIPS_CAST_HMAC_SHA2_512) != 0) {
+        err_sys("HMAC-SHA2-512 CAST failed");
+    }
+#endif
+#ifdef WOLFSSL_SHA3
+    if (wc_RunCast_fips(FIPS_CAST_HMAC_SHA3_256) != 0) {
+        err_sys("HMAC-SHA3-256 CAST failed");
+    }
+#endif
+#ifdef HAVE_HASHDRBG
+    if (wc_RunCast_fips(FIPS_CAST_DRBG) != 0) {
+        err_sys("Hash_DRBG CAST failed");
+    }
+#endif
+#ifndef NO_RSA
+    if (wc_RunCast_fips(FIPS_CAST_RSA_SIGN_PKCS1v15) != 0) {
+        err_sys("RSA sign CAST failed");
+    }
+#endif
+#if defined(HAVE_ECC_CDH) && defined(HAVE_ECC_CDH_CAST)
+    if (wc_RunCast_fips(FIPS_CAST_ECC_CDH) != 0) {
+        err_sys("RSA sign CAST failed");
+    }
+#endif
+#ifdef HAVE_ECC_DHE
+    if (wc_RunCast_fips(FIPS_CAST_ECC_PRIMITIVE_Z) != 0) {
+        err_sys("ECC Primitive Z CAST failed");
+    }
+#endif
+#ifdef HAVE_ECC
+    if (wc_RunCast_fips(FIPS_CAST_ECDSA) != 0) {
+        err_sys("ECDSA CAST failed");
+    }
+#endif
+#ifndef NO_DH
+    if (wc_RunCast_fips(FIPS_CAST_DH_PRIMITIVE_Z) != 0) {
+        err_sys("DH Primitive Z CAST failed");
+    }
+#endif
+#ifdef WOLFSSL_HAVE_PRF
+    if (wc_RunCast_fips(FIPS_CAST_KDF_TLS12) != 0) {
+        err_sys("KDF TLSv1.2 CAST failed");
+    }
+#endif
+#if defined(WOLFSSL_HAVE_PRF) && defined(WOLFSSL_TLS13)
+    if (wc_RunCast_fips(FIPS_CAST_KDF_TLS13) != 0) {
+        err_sys("KDF TLSv1.3 CAST failed");
+    }
+#endif
+#ifdef WOLFSSL_WOLFSSH
+    if (wc_RunCast_fips(FIPS_CAST_KDF_SSH) != 0) {
+        err_sys("KDF SSHv2.0 CAST failed");
+    }
+#endif
+#endif /* HAVE_FIPS && HAVE_FIPS_VERSION == 5 */
+#ifdef WOLFSSL_ALLOW_SKIP_UNIT_TESTS
+    if (argc == 1)
+#endif
+    {
+        ApiTest();
 
-    if ( (ret = HashTest()) != 0){
-        printf("hash test failed with %d\n", ret);
-        goto exit;
+        if ( (ret = HashTest()) != 0){
+            printf("hash test failed with %d\n", ret);
+            goto exit;
+        }
     }
 
+#ifndef NO_WOLFSSL_CIPHER_SUITE_TEST
 #if !defined(NO_WOLFSSL_CLIENT) && !defined(NO_WOLFSSL_SERVER)
 #ifndef SINGLE_THREADED
     if ( (ret = SuiteTest(argc, argv)) != 0){
@@ -88,6 +174,7 @@ int unit_test(int argc, char** argv)
     }
 #endif
 #endif
+#endif /* NO_WOLFSSL_CIPHER_SUITE_TEST */
 
     SrpTest();
 
