@@ -1,6 +1,6 @@
 /* cmac.c
  *
- * Copyright (C) 2006-2017 wolfSSL Inc.
+ * Copyright (C) 2006-2021 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -25,9 +25,22 @@
 #endif
 
 #include <wolfssl/wolfcrypt/settings.h>
+#ifdef WOLFSSL_QNX_CAAM
+#include <wolfssl/wolfcrypt/port/caam/wolfcaam.h>
+#endif
 
 #if defined(WOLFSSL_CMAC) && !defined(NO_AES) && defined(WOLFSSL_AES_DIRECT)
 
+#if defined(HAVE_FIPS) && defined(HAVE_FIPS_VERSION) && (HAVE_FIPS_VERSION >= 2)
+    /* set NO_WRAPPERS before headers, use direct internal f()s not wrappers */
+    #define FIPS_NO_WRAPPERS
+
+    #ifdef USE_WINDOWS_API
+        #pragma code_seg(".fipsA$n")
+        #pragma const_seg(".fipsB$n")
+    #endif
+#endif
+
 #ifdef NO_INLINE
     #include <wolfssl/wolfcrypt/misc.h>
 #else
@@ -39,8 +52,13 @@
 #include <wolfssl/wolfcrypt/aes.h>
 #include <wolfssl/wolfcrypt/cmac.h>
 
+#ifdef WOLF_CRYPTO_CB
+    #include <wolfssl/wolfcrypt/cryptocb.h>
+#endif
 
-static void ShiftAndXorRb(byte* out, byte* in)
+
+/* Used by AES-SIV. See aes.c. */
+void ShiftAndXorRb(byte* out, byte* in)
 {
     int i, j, xorRb;
     int mask = 0, last = 0;
@@ -59,36 +77,87 @@ static void ShiftAndXorRb(byte* out, byte* in)
     }
 }
 
-
-int wc_InitCmac(Cmac* cmac, const byte* key, word32 keySz,
-                int type, void* unused)
+/* returns 0 on success */
+int wc_InitCmac_ex(Cmac* cmac, const byte* key, word32 keySz,
+                int type, void* unused, void* heap, int devId)
 {
     int ret;
 
     (void)unused;
+    (void)heap;
 
-    if (cmac == NULL || key == NULL || keySz == 0 || type != WC_CMAC_AES)
+    if (cmac == NULL || keySz == 0 || type != WC_CMAC_AES) {
         return BAD_FUNC_ARG;
+    }
 
     XMEMSET(cmac, 0, sizeof(Cmac));
+
+#ifdef WOLF_CRYPTO_CB
+    if (devId != INVALID_DEVID) {
+        cmac->devId = devId;
+        cmac->devCtx = NULL;
+
+        ret = wc_CryptoCb_Cmac(cmac, key, keySz, NULL, 0, NULL, NULL,
+                type, unused);
+        if (ret != CRYPTOCB_UNAVAILABLE)
+            return ret;
+        /* fall-through when unavailable */
+    }
+#else
+    (void)devId;
+#endif
+
+    if (key == NULL) {
+        return BAD_FUNC_ARG;
+    }
+
     ret = wc_AesSetKey(&cmac->aes, key, keySz, NULL, AES_ENCRYPTION);
     if (ret == 0) {
         byte l[AES_BLOCK_SIZE];
 
         XMEMSET(l, 0, AES_BLOCK_SIZE);
-        wc_AesEncryptDirect(&cmac->aes, l, l);
-        ShiftAndXorRb(cmac->k1, l);
-        ShiftAndXorRb(cmac->k2, cmac->k1);
-        ForceZero(l, AES_BLOCK_SIZE);
+        ret = wc_AesEncryptDirect(&cmac->aes, l, l);
+        if (ret == 0) {
+            ShiftAndXorRb(cmac->k1, l);
+            ShiftAndXorRb(cmac->k2, cmac->k1);
+            ForceZero(l, AES_BLOCK_SIZE);
+        }
     }
     return ret;
 }
 
 
+int wc_InitCmac(Cmac* cmac, const byte* key, word32 keySz,
+                int type, void* unused)
+{
+#ifdef WOLFSSL_QNX_CAAM
+    int devId = WOLFSSL_CAAM_DEVID;
+#else
+    int devId = INVALID_DEVID;
+#endif
+    return wc_InitCmac_ex(cmac, key, keySz, type, unused, NULL, devId);
+}
+
+
+
 int wc_CmacUpdate(Cmac* cmac, const byte* in, word32 inSz)
 {
-    if ((cmac == NULL) || (in == NULL && inSz != 0))
+    int ret = 0;
+
+    if ((cmac == NULL) || (in == NULL && inSz != 0)) {
         return BAD_FUNC_ARG;
+    }
+
+#ifdef WOLF_CRYPTO_CB
+    if (cmac->devId != INVALID_DEVID) {
+        ret = wc_CryptoCb_Cmac(cmac, NULL, 0, in, inSz,
+                NULL, NULL, 0, NULL);
+        if (ret != CRYPTOCB_UNAVAILABLE)
+            return ret;
+        /* fall-through when unavailable */
+        ret = 0; /* reset error code */
+    }
+#endif
 
     while (inSz != 0) {
         word32 add = min(inSz, AES_BLOCK_SIZE - cmac->bufferSz);
@@ -99,29 +168,41 @@ int wc_CmacUpdate(Cmac* cmac, const byte* in, word32 inSz)
         inSz -= add;
 
         if (cmac->bufferSz == AES_BLOCK_SIZE && inSz != 0) {
-            if (cmac->totalSz != 0)
+            if (cmac->totalSz != 0) {
                 xorbuf(cmac->buffer, cmac->digest, AES_BLOCK_SIZE);
-            wc_AesEncryptDirect(&cmac->aes,
-                                cmac->digest,
-                                cmac->buffer);
-            cmac->totalSz += AES_BLOCK_SIZE;
-            cmac->bufferSz = 0;
+            }
+            ret = wc_AesEncryptDirect(&cmac->aes, cmac->digest, cmac->buffer);
+            if (ret == 0) {
+                cmac->totalSz += AES_BLOCK_SIZE;
+                cmac->bufferSz = 0;
+            }
         }
     }
 
-    return 0;
+    return ret;
 }
 
 
 int wc_CmacFinal(Cmac* cmac, byte* out, word32* outSz)
 {
+    int ret;
     const byte* subKey;
 
-    if (cmac == NULL || out == NULL)
+    if (cmac == NULL || out == NULL || outSz == NULL) {
         return BAD_FUNC_ARG;
-
-    if (outSz != NULL && *outSz < AES_BLOCK_SIZE)
+    }
+    if (*outSz < WC_CMAC_TAG_MIN_SZ || *outSz > WC_CMAC_TAG_MAX_SZ) {
         return BUFFER_E;
+    }
+
+#ifdef WOLF_CRYPTO_CB
+    if (cmac->devId != INVALID_DEVID) {
+        ret = wc_CryptoCb_Cmac(cmac, NULL, 0, NULL, 0, out, outSz, 0, NULL);
+        if (ret != CRYPTOCB_UNAVAILABLE)
+            return ret;
+        /* fall-through when unavailable */
+    }
+#endif
 
     if (cmac->bufferSz == AES_BLOCK_SIZE) {
         subKey = cmac->k1;
@@ -129,23 +210,26 @@ int wc_CmacFinal(Cmac* cmac, byte* out, word32* outSz)
     else {
         word32 remainder = AES_BLOCK_SIZE - cmac->bufferSz;
 
-        if (remainder == 0)
+        if (remainder == 0) {
             remainder = AES_BLOCK_SIZE;
-
-        if (remainder > 1)
+        }
+        if (remainder > 1) {
             XMEMSET(cmac->buffer + AES_BLOCK_SIZE - remainder, 0, remainder);
+        }
         cmac->buffer[AES_BLOCK_SIZE - remainder] = 0x80;
         subKey = cmac->k2;
     }
     xorbuf(cmac->buffer, cmac->digest, AES_BLOCK_SIZE);
     xorbuf(cmac->buffer, subKey, AES_BLOCK_SIZE);
-    wc_AesEncryptDirect(&cmac->aes, out, cmac->buffer);
+    ret = wc_AesEncryptDirect(&cmac->aes, cmac->digest, cmac->buffer);
+    if (ret == 0) {
+        XMEMCPY(out, cmac->digest, *outSz);
+    }
 
-    if (outSz != NULL)
-        *outSz = AES_BLOCK_SIZE;
+    wc_AesFree(&cmac->aes);
     ForceZero(cmac, sizeof(Cmac));
 
-    return 0;
+    return ret;
 }
 
 
@@ -153,25 +237,39 @@ int wc_AesCmacGenerate(byte* out, word32* outSz,
                        const byte* in, word32 inSz,
                        const byte* key, word32 keySz)
 {
-    Cmac cmac;
     int ret;
+#ifdef WOLFSSL_SMALL_STACK
+    Cmac *cmac;
+#else
+    Cmac cmac[1];
+#endif
 
-    if (out == NULL || (in == NULL && inSz > 0) || key == NULL || keySz == 0)
+    if (out == NULL || (in == NULL && inSz > 0) || key == NULL || keySz == 0) {
         return BAD_FUNC_ARG;
+    }
 
-    ret = wc_InitCmac(&cmac, key, keySz, WC_CMAC_AES, NULL);
-    if (ret != 0)
-        return ret;
+#ifdef WOLFSSL_SMALL_STACK
+    if ((cmac = (Cmac *)XMALLOC(sizeof *cmac, NULL,
+                                DYNAMIC_TYPE_CMAC)) == NULL) {
+        return MEMORY_E;
+    }
+#endif
 
-    ret = wc_CmacUpdate(&cmac, in, inSz);
-    if (ret != 0)
-        return ret;
+    ret = wc_InitCmac(cmac, key, keySz, WC_CMAC_AES, NULL);
+    if (ret == 0) {
+        ret = wc_CmacUpdate(cmac, in, inSz);
+    }
+    if (ret == 0) {
+        ret = wc_CmacFinal(cmac, out, outSz);
+    }
 
-    ret = wc_CmacFinal(&cmac, out, outSz);
-    if (ret != 0)
-        return ret;
+#ifdef WOLFSSL_SMALL_STACK
+    if (cmac) {
+        XFREE(cmac, NULL, DYNAMIC_TYPE_CMAC);
+    }
+#endif
 
-    return 0;
+    return ret;
 }
 
 
@@ -179,24 +277,24 @@ int wc_AesCmacVerify(const byte* check, word32 checkSz,
                      const byte* in, word32 inSz,
                      const byte* key, word32 keySz)
 {
+    int ret;
     byte a[AES_BLOCK_SIZE];
     word32 aSz = sizeof(a);
-    int result;
     int compareRet;
 
     if (check == NULL || checkSz == 0 || (in == NULL && inSz != 0) ||
-        key == NULL || keySz == 0)
-
+        key == NULL || keySz == 0) {
         return BAD_FUNC_ARG;
+    }
 
     XMEMSET(a, 0, aSz);
-    result = wc_AesCmacGenerate(a, &aSz, in, inSz, key, keySz);
+    ret = wc_AesCmacGenerate(a, &aSz, in, inSz, key, keySz);
     compareRet = ConstantCompare(check, a, min(checkSz, aSz));
 
-    if (result == 0)
-        result = compareRet ? 1 : 0;
+    if (ret == 0)
+        ret = compareRet ? 1 : 0;
 
-    return result;
+    return ret;
 }
 
 
